@@ -64,32 +64,10 @@ enum class VersionNumber : uint32_t {
   SPIRV_1_3 = 0x00010300,
   SPIRV_1_4 = 0x00010400,
   SPIRV_1_5 = 0x00010500,
-  SPIRV_1_6 = 0x00010600,
   MinimumVersion = SPIRV_1_0,
-  MaximumVersion = SPIRV_1_6
+  MaximumVersion = SPIRV_1_5
 };
 
-inline constexpr std::string_view formatVersionNumber(uint32_t Version) {
-  switch (Version) {
-  case static_cast<uint32_t>(VersionNumber::SPIRV_1_0):
-    return "1.0";
-  case static_cast<uint32_t>(VersionNumber::SPIRV_1_1):
-    return "1.1";
-  case static_cast<uint32_t>(VersionNumber::SPIRV_1_2):
-    return "1.2";
-  case static_cast<uint32_t>(VersionNumber::SPIRV_1_3):
-    return "1.3";
-  case static_cast<uint32_t>(VersionNumber::SPIRV_1_4):
-    return "1.4";
-  }
-  return "unknown";
-}
-
-inline bool isSPIRVVersionKnown(uint32_t Ver) {
-  return Ver >= static_cast<uint32_t>(VersionNumber::MinimumVersion) &&
-         Ver <= static_cast<uint32_t>(VersionNumber::MaximumVersion);
-}
-
 enum class ExtensionID : uint32_t {
   First,
 #define EXT(X) X,
@@ -223,14 +201,6 @@ public:
     PreserveOCLKernelArgTypeMetadataThroughString = Value;
   }
 
-  bool shouldEmitFunctionPtrAddrSpace() const noexcept {
-    return EmitFunctionPtrAddrSpace;
-  }
-
-  void setEmitFunctionPtrAddrSpace(bool Value) noexcept {
-    EmitFunctionPtrAddrSpace = Value;
-  }
-
   void setBuiltinFormat(BuiltinFormat Value) noexcept {
     SPIRVBuiltinFormat = Value;
   }
@@ -277,10 +247,6 @@ private:
   // kernel_arg_type_qual metadata through OpString
   bool PreserveOCLKernelArgTypeMetadataThroughString = false;
 
-  // Controls if CodeSectionINTEL can be emitted and consumed with a dedicated
-  // address space
-  bool EmitFunctionPtrAddrSpace = false;
-
   bool PreserveAuxData = false;
 
   BuiltinFormat SPIRVBuiltinFormat = BuiltinFormat::Function;
